@@ -11,9 +11,7 @@
 # The above copyright notice and this permission notice shall be
 # included in all copies or substantial portions of the Software.
 
-"""Adds support for ipfs. Requires go-ipfs and a running ipfs daemon
-"""
-
+"""Adds support for ipfs. Requires go-ipfs and a running ipfs daemon"""
 
 import os
 import shutil
@@ -22,7 +20,6 @@ import tempfile
 
 from beets import config, library, ui, util
 from beets.plugins import BeetsPlugin
-from beets.util import syspath
 
 
 class IPFSPlugin(BeetsPlugin):
@@ -77,29 +74,29 @@ class IPFSPlugin(BeetsPlugin):
 
         def func(lib, opts, args):
             if opts.add:
-                for album in lib.albums(ui.decargs(args)):
+                for album in lib.albums(args):
                     if len(album.items()) == 0:
                         self._log.info(
-                            "{0} does not contain items, aborting", album
+                            "{} does not contain items, aborting", album
                         )
 
                     self.ipfs_add(album)
                     album.store()
 
             if opts.get:
-                self.ipfs_get(lib, ui.decargs(args))
+                self.ipfs_get(lib, args)
 
             if opts.publish:
                 self.ipfs_publish(lib)
 
             if opts._import:
-                self.ipfs_import(lib, ui.decargs(args))
+                self.ipfs_import(lib, args)
 
             if opts._list:
-                self.ipfs_list(lib, ui.decargs(args))
+                self.ipfs_list(lib, args)
 
             if opts.play:
-                self.ipfs_play(lib, opts, ui.decargs(args))
+                self.ipfs_play(lib, opts, args)
 
         cmd.func = func
         return [cmd]
@@ -125,13 +122,13 @@ class IPFSPlugin(BeetsPlugin):
             return False
         try:
             if album.ipfs:
-                self._log.debug("{0} already added", album_dir)
+                self._log.debug("{} already added", album_dir)
                 # Already added to ipfs
                 return False
         except AttributeError:
             pass
 
-        self._log.info("Adding {0} to ipfs", album_dir)
+        self._log.info("Adding {} to ipfs", album_dir)
 
         if self.config["nocopy"]:
             cmd = "ipfs add --nocopy -q -r".split()
@@ -141,7 +138,7 @@ class IPFSPlugin(BeetsPlugin):
         try:
             output = util.command_output(cmd).stdout.split()
         except (OSError, subprocess.CalledProcessError) as exc:
-            self._log.error("Failed to add {0}, error: {1}", album_dir, exc)
+            self._log.error("Failed to add {}, error: {}", album_dir, exc)
             return False
         length = len(output)
 
@@ -149,12 +146,12 @@ class IPFSPlugin(BeetsPlugin):
             line = line.strip()
             if linenr == length - 1:
                 # last printed line is the album hash
-                self._log.info("album: {0}", line)
+                self._log.info("album: {}", line)
                 album.ipfs = line
             else:
                 try:
                     item = album.items()[linenr]
-                    self._log.info("item: {0}", line)
+                    self._log.info("item: {}", line)
                     item.ipfs = line
                     item.store()
                 except IndexError:
@@ -183,11 +180,11 @@ class IPFSPlugin(BeetsPlugin):
             util.command_output(cmd)
         except (OSError, subprocess.CalledProcessError) as err:
             self._log.error(
-                "Failed to get {0} from ipfs.\n{1}", _hash, err.output
+                "Failed to get {} from ipfs.\n{.output}", _hash, err
             )
             return False
 
-        self._log.info("Getting {0} from ipfs", _hash)
+        self._log.info("Getting {} from ipfs", _hash)
         imp = ui.commands.TerminalImportSession(
             lib, loghandler=None, query=None, paths=[_hash]
         )
@@ -195,7 +192,7 @@ class IPFSPlugin(BeetsPlugin):
         # This uses a relative path, hence we cannot use util.syspath(_hash,
         # prefix=True). However, that should be fine since the hash will not
         # exceed MAX_PATH.
-        shutil.rmtree(syspath(_hash, prefix=False))
+        shutil.rmtree(util.syspath(_hash, prefix=False))
 
     def ipfs_publish(self, lib):
         with tempfile.NamedTemporaryFile() as tmp:
@@ -211,7 +208,7 @@ class IPFSPlugin(BeetsPlugin):
                 msg = f"Failed to publish library. Error: {err}"
                 self._log.error(msg)
                 return False
-            self._log.info("hash of library: {0}", output)
+            self._log.info("hash of library: {}", output)
 
     def ipfs_import(self, lib, args):
         _hash = args[0]
@@ -235,7 +232,7 @@ class IPFSPlugin(BeetsPlugin):
             try:
                 util.command_output(cmd)
             except (OSError, subprocess.CalledProcessError):
-                self._log.error(f"Could not import {_hash}")
+                self._log.error("Could not import {}", _hash)
                 return False
 
         # add all albums from remotes into a combined library
@@ -301,9 +298,7 @@ class IPFSPlugin(BeetsPlugin):
                     break
             except AttributeError:
                 pass
-            item_path = os.path.basename(item.path).decode(
-                util._fsencoding(), "ignore"
-            )
+            item_path = os.fsdecode(os.path.basename(item.path))
             # Clear current path from item
             item.path = f"/ipfs/{album.ipfs}/{item_path}"
 
@@ -311,7 +306,7 @@ class IPFSPlugin(BeetsPlugin):
             items.append(item)
         if len(items) < 1:
             return False
-        self._log.info("Adding '{0}' to temporary library", album)
+        self._log.info("Adding '{}' to temporary library", album)
         new_album = tmplib.add_album(items)
         new_album.ipfs = album.ipfs
         new_album.store(inherit=False)
