@@ -130,9 +130,6 @@ class SpotifyPlugin(
                 "mode": "list",
                 "tiebreak": "popularity",
                 "show_failures": False,
-                "artist_field": "albumartist",
-                "album_field": "album",
-                "track_field": "title",
                 "region_filter": None,
                 "regex": [],
                 "client_id": "4e414367a1d14c75a5c5129a627fcab8",
@@ -563,13 +560,17 @@ class SpotifyPlugin(
                     regex["search"], regex["replace"], value
                 )
 
-            # Custom values can be passed in the config (just in case)
-            artist = item[self.config["artist_field"].get()]
-            album = item[self.config["album_field"].get()]
-            query_string = item[self.config["track_field"].get()]
+            artist = item["artist"] or item["albumartist"]
+            album = item["album"]
+            query_string = item["title"]
 
             # Query the Web API for each track, look for the items' JSON data
-            query_filters: SearchFilter = {"artist": artist, "album": album}
+            query_filters: SearchFilter = {}
+            if artist:
+                query_filters["artist"] = artist
+            if album:
+                query_filters["album"] = album
+
             response_data_tracks = self._search_api(
                 query_type="track",
                 query_string=query_string,
@@ -693,12 +694,10 @@ class SpotifyPlugin(
             audio_features = self.track_audio_features(spotify_track_id)
             if audio_features is None:
                 self._log.info("No audio features found for: {}", item)
-                continue
-            for feature in audio_features.keys():
-                if feature in self.spotify_audio_features.keys():
-                    item[self.spotify_audio_features[feature]] = audio_features[
-                        feature
-                    ]
+            else:
+                for feature, value in audio_features.items():
+                    if feature in self.spotify_audio_features:
+                        item[self.spotify_audio_features[feature]] = value
             item["spotify_updated"] = time.time()
             item.store()
             if write:
